<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2013 Gerald Loß <gerald.loss@gmx.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * Plugin for the 'glcrossword' extension.
 *
 * @author	Gerald Loß <gerald.loss@gmx.de>
 * @package	TYPO3
 * @subpackage	tx_glcrossword
 */
//class tx_glcrossword_pi1 extends tslib_pibase {
class tx_glcrossword_pi1 extends \TYPO3\CMS\Frontend\Plugin\AbstractPlugin {
	public $prefixId      = 'tx_glcrossword_pi1';		// Same as class name
	public $scriptRelPath = 'pi1/class.tx_glcrossword_pi1.php';	// Path to this script relative to the extension dir.
	public $extKey        = 'glcrossword';	// The extension key.
	// deactivate cacheing for this plugin. Otherwise the session is not correct written
	// and wie got no data in the ajax requests
	public $pi_checkCHash = FALSE;

	/**
	 * Database table with the questions.
	 * @access protected
	 * @var string
	 */
	const C_STR_DB_TABLE_QUESTIONS = 'tx_glcrossword_questions';
	
	/**
	 * Instanz of the xajax manager
	 * @access protected
	 * @var tx_xajax 
	 */
	protected $m_objXajax;
	
	/**
	 * The crossword object.
	 * @var tx_glcrossword_crossword
	 */
	protected $m_objCrossword;
	
	/**
	 * The main method of the Plugin.
	 *
	 * @param string $content The Plugin content
	 * @param array $conf The Plugin configuration
	 * @return string The content that is displayed on the website
	 */ 
	public function main($content, array $conf) {
		
		// the height and the width of the crossword
		$l_intHeightOfCrossword = 0;
		$l_intWidthOfCrossword = 0;
		// the UIDs of the related questions
		$l_strRelatedQuestions = ''.
		// the X scale of the crossword size
		$l_fltXScale = 0;
		// the Y scale of the crossword size
		$l_fltYScale = 0;
		// the borderwidth of the crossword
		$l_intBorderWidth = 0;
		// the unique ID of this content element
		$l_intUniqueId = $this->cObj->data['uid'];
		// location of the alternate css file
		$l_strAlternateCssFile = '';
		// title of the crossword
		$l_strCrosswordTitle = '';
		
		// deactivate cacheing for this plugin. Otherwise the session is not correct written
		// and wie got no data in the ajax requests
		$this->pi_USER_INT_obj = 1; 
		
		$this->conf = $conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		
		// transform flexform into an array
		$this->pi_initPIflexForm();
		
		// read the title of the crossword
		$l_strCrosswordTitle = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'titleOfCrossword');
		// read the width of the crossword
		$l_intWidthOfCrossword = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'widthOfCrossword');
		// read the height of the crossword
		$l_intHeightOfCrossword = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'heightOfCrossword');

		// read the PIDs of the related questions
		$l_strRelatedQuestions = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'relatetQuestions', 'sQUESTIONS');
		
		// read the scale factor for the width of the crossword
		$l_fltXScale = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'scaleXFactor');
		// read the scale factor for the height of the crossword
		$l_fltYScale = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'scaleYFactor');
		// read the border width of the crossword boxes
		$l_intBorderWidth = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'borderWidth');
		
		// create a crossword object
		$this->m_objCrossword = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( 'tx_glcrossword_crossword',
														 $l_strCrosswordTitle,
														 $l_intUniqueId,
														 $l_intHeightOfCrossword,
														 $l_intWidthOfCrossword,
														 $l_fltXScale,
														 $l_fltYScale,
														 $l_intBorderWidth,
														 $l_strRelatedQuestions,
														 $this );
		
		// read the alternate CSS file
		$l_strAlternateCssFile = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'cssFile');
		
		// set the javascript content
		$this->setJsContent($l_intUniqueId, $l_strAlternateCssFile, $l_strCrosswordTitle);
		
		// return content
		$content .= $this->m_objCrossword->draw();

		// write the array with all crosswords back into the session, for later access over the ajax connection
		$GLOBALS['TSFE']->fe_user->setAndSaveSessionData('glcrossword_arrCrosswords', tx_glcrossword_crossword::$m_arrCrosswords);
		
		return $this->pi_wrapInBaseClass($content);
	}
	 
	
	/**
	 * Returns the content for the javascript libraries
	 * @param 	integer $i_intUniqueId 			The unique ID of the content element.
	 * @param 	string	$i_strAlternateCssFile	Path of the alternate CSS file.
	 * @param 	string	$i_strCrosswordTitle	Title of the crossword.
	 */
	protected function setJsContent($i_intUniqueId, $i_strAlternateCssFile, $i_strCrosswordTitle) {
		// the returning content
		$l_strContent = '';
		// path of the glcrossword css file
		$l_strCssPath = '';
		
		// if not already exists
		// or additiona header data is empty
		if (strpos($GLOBALS['TSFE']->additionalHeaderData['glcrossword'], 'jquery-1.10.2.min.js') === false
					|| $GLOBALS['TSFE']->additionalHeaderData['glcrossword'] == null) {
			// insert haeder data with the javascript jQuery library
			$GLOBALS['TSFE']->additionalHeaderData['glcrossword'] .= '<!-- The jQuery source file -->' . "\n";
			$GLOBALS['TSFE']->additionalHeaderData['glcrossword'] .= 
				'<script src="' . t3lib_extMgm::extRelPath('glcrossword') . 'js/jquery-1.10.2.min.js" type="text/javascript"></script>' . "\n";
		}
		
		// if not already exists
		// or additiona header data is empty
		if (strpos($GLOBALS['TSFE']->additionalHeaderData['glcrossword'], 'jquery.qtip.min.js') === false
					|| $GLOBALS['TSFE']->additionalHeaderData['glcrossword'] == null) {
			// insert the jquery qtip� library	
			$GLOBALS['TSFE']->additionalHeaderData['glcrossword'] .= '<!-- The jQuery qtip� source and css file -->' . "\n";
			$GLOBALS['TSFE']->additionalHeaderData['glcrossword'] .=
				'<script src="' . t3lib_extMgm::extRelPath('glcrossword') . 'js/jquery.qtip.min.js" type="text/javascript"></script>' . "\n";
			$GLOBALS['TSFE']->additionalHeaderData['glcrossword'] .=
				'<link rel="stylesheet" type="text/css" href="' . t3lib_extMgm::extRelPath('glcrossword') . 'css/jquery.qtip.min.css" media="all" />' . "\n";
		}
				
		// if not already exists
		// or additiona header data is empty
		if (strpos($GLOBALS['TSFE']->additionalHeaderData['glcrossword'], 'jquery-ui-1.10.3.custom.min.js') === false
					|| $GLOBALS['TSFE']->additionalHeaderData['glcrossword'] == null) {
			// insert the jquery-ui library	
			$GLOBALS['TSFE']->additionalHeaderData['glcrossword'] .= '<!-- The jQuery ui source and css file -->' . "\n";
			$GLOBALS['TSFE']->additionalHeaderData['glcrossword'] .=
				'<script src="' . t3lib_extMgm::extRelPath('glcrossword') . 'js/jquery-ui-1.10.3.custom.min.js" type="text/javascript"></script>' . "\n";
			$GLOBALS['TSFE']->additionalHeaderData['glcrossword'] .=
				'<link rel="stylesheet" type="text/css" href="' . t3lib_extMgm::extRelPath('glcrossword') . 'css/jquery-ui-1.10.3.custom.min.css" media="all" />' . "\n";
		}
				
		// if not already exists
		// or additiona header data is empty
		if (strpos($GLOBALS['TSFE']->additionalHeaderData['glcrossword'], 'glcrossword.min.js') === false
					|| $GLOBALS['TSFE']->additionalHeaderData['glcrossword'] == null) {
			// insert haeder data with the javascript gcrossword library
			$GLOBALS['TSFE']->additionalHeaderData['glcrossword'] .= '<!-- The main GLCrossword source file -->' . "\n";
			$GLOBALS['TSFE']->additionalHeaderData['glcrossword'] .=
				'<script src="' . t3lib_extMgm::extRelPath('glcrossword') . 'js/glcrossword.min.js" type="text/javascript"></script>' . "\n";
		}
				
		// if an alternative CSS file is given
		if ($i_strAlternateCssFile != '') {
			// if the alternative CSS file starts with /
			if (substr($i_strAlternateCssFile, 0, 1) == '/') {
				$l_strCssPath = 'fileadmin' . $i_strAlternateCssFile; 
			}
			// if the alternative CSS file starts without /
			else {
				$l_strCssPath = 'fileadmin/' . $i_strAlternateCssFile; 
			}
		}
		// if no alternative CSS file is given
		else {
			$l_strCssPath = t3lib_extMgm::extRelPath('glcrossword') . 'css/glcrossword.css';
		}
		
		// if not already exists
		// or additiona header data is empty
		if (strpos($GLOBALS['TSFE']->additionalHeaderData['glcrossword'], $l_strCssPath) === false
					|| $GLOBALS['TSFE']->additionalHeaderData['glcrossword'] == null) {
		
			// insert css file of glcrossword
			$GLOBALS['TSFE']->additionalHeaderData['glcrossword'] .= '<!-- The css file of glcrossword -->' . "\n";
			$GLOBALS['TSFE']->additionalHeaderData['glcrossword'] .=
			'<link rel="stylesheet" type="text/css" href="' . $l_strCssPath . '" media="all" />' . "\n";
		}
		
		// insert the start of the main function
		$l_strContent = "\n" . '<script type="text/javascript">' . "\n"; 
		$l_strContent .= "\n\t" . '// init header script for crossword "' . $i_strCrosswordTitle . '" with ID ' . $i_intUniqueId; 
		$l_strContent .= "\n\t" . 'if (arrGlcrosswordIds == null) {
		
		var arrGlcrosswordIds = new Array("' . $i_intUniqueId . '");
		// if document ready, call the init function of the crossword
		jQuery( document ).on(
			"ready",
			{ intUniqueIds: arrGlcrosswordIds },
			function( objEvent ){
				GlCrossword.glcrosswordInit(objEvent);
			}
		);
	} else {
		arrGlcrosswordIds.push( "' . $i_intUniqueId . '" );
	}';
		
		$l_strContent .= "\n" . '</script>' . "\n\n";
		
		// write this content into the header
		$GLOBALS['TSFE']->additionalHeaderData['glcrossword'] .= $l_strContent;
	}
}



if (defined('TYPO3_MODE') && isset($GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['ext/glcrossword/pi1/class.tx_glcrossword_pi1.php'])) {
	include_once($GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['ext/glcrossword/pi1/class.tx_glcrossword_pi1.php']);
}

?>